@@ -42,12 +42,12 @@
 #include <glib/gstdio.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <gnome.h>
 #include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
-#include <libgnome/libgnometypebuiltins.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
+#include <bonobo-activation/bonobo-activation.h>
 
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -65,6 +65,7 @@
 #define ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
 #define GTK_THEME_KEY	"/desktop/gnome/interface/gtk_theme"
 #define ICON_THEME_KEY  "/desktop/gnome/interface/icon_theme"
+#define GTK_MODULES_KEY "/apps/gnome_settings_daemon/gtk-modules"
 #define DESKTOP_IS_HOME_DIR "/apps/nautilus/preferences/desktop_is_home_dir"
 #define MIN_REPORT_DETAILS_CHARS 10
 
@@ -72,12 +73,27 @@ static GOptionData gopt_data;
 static int bug_count = 0;
 static GHashTable *apps = NULL;
 
+static gboolean
+show_version_cb (const char *option_name,
+                 const char *value,
+                 gpointer data,
+                 GError **error)
+{
+	g_print ("%s %s\n", _("GNOME bug-buddy"), VERSION);
+
+	exit (EXIT_SUCCESS);
+
+	return FALSE;
+}
+
 static const GOptionEntry options[] = {
 	{ "package",    '\0', 0, G_OPTION_ARG_STRING,   &gopt_data.package,       N_("Package containing the program"),          N_("PACKAGE") },
 	{ "appname",    '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.app_file,      N_("File name of crashed program"),            N_("FILE") },
 	{ "pid",        '\0', 0, G_OPTION_ARG_INT,      &gopt_data.pid,           N_("PID of crashed program"),                  N_("PID") },
 	{ "include",    '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.include_file,  N_("Text file to include in the report"),      N_("FILE") },
+	{ "unlink-tempfile", '\0', 0, G_OPTION_ARG_NONE, &gopt_data.own_file,     N_("Delete the included file after reporting"),NULL},
 	{ "minidump",   '\0', 0, G_OPTION_ARG_FILENAME, &gopt_data.minidump_file, N_("MiniDump file with info about the crash"), N_("FILE") },
+	{ "version", '\0', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, show_version_cb, NULL, NULL },
 	{ NULL }
 };
 
@@ -85,6 +101,7 @@ static void fill_stderr_info (GtkBuilder *ui);
 static void fill_custom_info (BugzillaApplication *app, GtkBuilder *ui);
 static void close_callback   (GtkWidget *widget, gpointer user_data);
 static void bug_buddy_quit   (GtkBuilder *ui);
+static void fill_include_file (char *filename, gboolean own_file, GtkBuilder *ui);
 
 
 static void
@@ -276,11 +293,8 @@ get_gnome_version_info (void)
 
 	version = g_new0 (GnomeVersionInfo, 1);
 
-	xml_file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR,
-					      "gnome-about/gnome-version.xml",
-					      TRUE, NULL);
-	if (!xml_file)
-		return NULL;
+	xml_file = g_build_filename (DATADIR, "gnome-about/gnome-version.xml", NULL);
+	
 	doc = xmlParseFile (xml_file);
 	g_free (xml_file);
 
@@ -288,7 +302,7 @@ get_gnome_version_info (void)
 		return NULL;
 
 	platform = minor = micro = distributor = date = NULL;
-	
+
 	for (node = xmlDocGetRootElement (doc)->children; node; node = node->next) {
 		if (!strcmp ((char *)node->name, "platform"))
 			platform = xmlNodeGetContent (node);
@@ -395,7 +409,6 @@ static void
 link_callback (GtkLinkButton *button, gpointer user_data)
 {
 	const gchar *link = gtk_link_button_get_uri (button);
-#if GTK_CHECK_VERSION (2,13,0)
 	GdkAppLaunchContext *context;
 
 	context = gdk_app_launch_context_new ();
@@ -407,10 +420,7 @@ link_callback (GtkLinkButton *button, gpointer user_data)
 	if (!g_app_info_launch_default_for_uri (link,
 						G_APP_LAUNCH_CONTEXT (context),
 						NULL))
-#else
-	if (!g_app_info_launch_default_for_uri (link, NULL, NULL))
-#endif
-    {
+	{
 		char *text;
 
 		text = g_markup_printf_escaped (_("Bug Buddy was unable to view the link \"%s\"\n"), link);
@@ -418,9 +428,7 @@ link_callback (GtkLinkButton *button, gpointer user_data)
 		g_free (text);
 	}
 
-#if GTK_CHECK_VERSION (2,13,0)
 	g_object_unref (context);
-#endif
 
 	return;
 }
@@ -779,7 +787,7 @@ send_report (BugzillaApplication *app, GnomeVersionInfo *gnome_version, GtkBuild
 	gtk_text_buffer_get_end_iter (buffer, &end);
 	details_text = gtk_text_buffer_get_text (buffer, &start, &end, FALSE);
 
-	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT(ui), "type"));
+	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (ui), "type"));
 	final_text = g_strdup_printf ("%s%s\n\n\n%s", 
 				      type == BUG_TYPE_CRASH ? "What were you doing when the application crashed?\n" : "",
 				      details_text != NULL ? details_text : "",
@@ -1057,6 +1065,12 @@ on_send_clicked (GtkWidget *button, gpointer data)
 			return;
 	}
 
+	/* hide the send button immediately so that the user can't click
+	 * it more than once (this will create multiple bugs).
+	 */
+	gtk_widget_hide (GTK_WIDGET (
+			gtk_builder_get_object (ui, "send-button")));
+
 	gtk_widget_show (GTK_WIDGET (
 			gtk_builder_get_object (ui, "progressbar")));
 	gtk_widget_hide (GTK_WIDGET (
@@ -1371,6 +1385,11 @@ unknown_app_finished (GtkBuilder *ui)
 {
 	GtkWidget *button;
 	char *label_text;
+	
+	/* add the include file information now that gdb has run */
+	if (gopt_data.include_file != NULL) {
+		fill_include_file (gopt_data.include_file, gopt_data.own_file, ui);
+	}
 
 	fill_stderr_info (ui);
 
@@ -1465,16 +1484,14 @@ close_callback (GtkWidget *widget, gpointer user_data)
 static void
 help_callback (GtkWidget *widget, gpointer user_data)
 {
-	GnomeProgram *program = (GnomeProgram *)user_data;
 	GError *error = NULL;
+	gboolean res;
 
-	gnome_help_display_desktop (program, 
-	                            "user-guide",
-	                            "user-guide", 
-	                            "feedback-bugs",
-	                            &error);
+	res = gtk_show_uri (gtk_widget_get_screen (widget),
+			    "ghelp:user-guide#feedback-bugs",
+			    gtk_get_current_event_time (), &error);
 
-	if (error) {
+	if (!res) {
 		GtkWidget *error_dialog = 
 		    gtk_message_dialog_new (NULL, 
 		                            GTK_DIALOG_MODAL,
@@ -1548,6 +1565,11 @@ fill_custom_info (BugzillaApplication *app, GtkBuilder *ui)
 	g_return_if_fail (app != NULL);
 	g_return_if_fail (ui != NULL);
 
+	/* fill the include info now that gdb has run */
+	if (gopt_data.include_file != NULL) {
+		fill_include_file (gopt_data.include_file, gopt_data.own_file, ui);
+	}
+
 	if (app->extra_info_script == NULL) {
 		return;
 	}
@@ -1611,20 +1633,33 @@ fill_proccess_info (pid_t pid, GtkBuilder *ui)
 }
 
 static void
-fill_include_file (char *filename, GtkBuilder *ui)
+fill_include_file (char *filename, gboolean own_file, GtkBuilder *ui)
 {
 	GtkTextView *text_view;
 	GtkTextIter end;
 	GtkTextBuffer *buffer;
 	char *text;
+	gboolean res;
 	GError *error = NULL;
 
-	if (g_file_get_contents (filename, &text, NULL, &error) != TRUE) {
+	res = g_file_get_contents (filename, &text, NULL, &error);
+
+	/* if own_file is set, delete the file regardless of the result:
+	 * if _get_contents () fails chances are the file has already been
+	 * deleted anyway.
+ 	 */
+
+	if (own_file) {
+		if (g_unlink (filename) == -1)
+			g_warning ("Unable to delete %s", filename);
+	}
+
+	if (!res) {
 		buddy_error (NULL, error->message);
 		g_error_free (error);
 		return;
 	}
-	
+
 	text_view = GTK_TEXT_VIEW (gtk_builder_get_object (ui, "gdb-text"));
 	buffer = gtk_text_view_get_buffer (text_view);
 	gtk_text_buffer_get_end_iter (buffer, &end);
@@ -1634,6 +1669,20 @@ fill_include_file (char *filename, GtkBuilder *ui)
 
 }
 
+/* copied from gconf-editor/gconf-util.c */
+
+static gchar *
+gconf_get_key_name_from_path (const gchar *path)
+{
+	const gchar *ptr;
+
+	ptr = path + strlen (path);
+
+	while (ptr[-1] != '/')
+		ptr--;
+
+	return g_strdup (ptr);
+}
 
 static void
 fill_system_info (GtkBuilder *ui)
@@ -1642,7 +1691,8 @@ fill_system_info (GtkBuilder *ui)
 	GtkTextView *text_view;
 	GtkTextIter end;
 	GtkTextBuffer *buffer;
-	GString *system_info;
+	GString *system_info, *modules;
+	GSList *entries;
 	struct utsname uts_buf;
 	char *str;
 	gboolean has_selinux, enforcing, a11y;
@@ -1650,6 +1700,7 @@ fill_system_info (GtkBuilder *ui)
 	g_return_if_fail (ui != NULL);
 	
 	system_info = g_string_new ("");
+	modules = NULL;
 
 	if (uname (&uts_buf) == 0) {
 		g_string_append_printf (system_info, "System: %s %s %s %s\n", uts_buf.sysname, uts_buf.release, uts_buf.version, uts_buf.machine);
@@ -1687,6 +1738,77 @@ fill_system_info (GtkBuilder *ui)
 	str = gconf_client_get_string (gconf_client, ICON_THEME_KEY, NULL);
 	g_string_append_printf (system_info, "Icon Theme: %s\n", str);
 	g_free (str);
+
+	/* add the GTK+ loaded modules. to do that, we look both in GConf and
+	 * inside the GTK_MODULES env var.
+	 */
+
+	entries = gconf_client_all_entries (gconf_client, GTK_MODULES_KEY, NULL);
+	if (entries) {
+		GSList *l;
+		GConfEntry *entry;
+		char *name;
+		GConfValue *val;
+
+		modules = g_string_new ("GTK+ Modules: ");
+
+		for (l = entries; l != NULL; l = l->next)  {
+			entry = l->data;
+			val = gconf_entry_get_value (entry);
+
+			/* if the value is a boolean, check if it's activated, otherwise if it's
+			 * a string, it will refer to a boolean; check that.
+			 * i know this is quite ugly.
+			 */
+			if ((val->type == GCONF_VALUE_BOOL && gconf_value_get_bool (val)) ||
+			    (val->type == GCONF_VALUE_STRING && gconf_client_get_bool (gconf_client,
+										       gconf_value_get_string (val),
+										       NULL)))
+			{
+
+				name = gconf_get_key_name_from_path (gconf_entry_get_key (entry));
+
+				if (!g_strstr_len (modules->str, modules->len, name)) {
+					g_string_append_printf (modules, "%s, ", name);
+				}
+
+				g_free (name);
+			}
+
+			gconf_entry_unref (entry);
+		}
+
+		g_slist_free (entries);
+	}
+
+	str = (char * ) g_getenv ("GTK_MODULES");
+	if (str) {
+		char **splitted;
+		int i;
+
+		if (!modules) {
+			modules = g_string_new ("GTK+ Modules: ");
+		}
+
+		/* modules are divided by G_SEARCHPATH_SEPARATOR */
+		splitted = pango_split_file_list (str);
+
+		for (i = 0; splitted[i]; i++) {
+			if (!g_strstr_len (modules->str, modules->len, splitted[i])) {
+				g_string_append_printf (modules, "%s, ", splitted[i]);
+			}
+		}
+
+		g_strfreev (splitted);
+	}
+
+	if (modules) {
+		/* discard the last ", " */
+		g_string_append_len (system_info, modules->str, (modules->len - 2));
+		g_string_append (system_info, "\n");
+		g_string_free (modules, TRUE);
+	}
+
 	g_object_unref (gconf_client);
 
 	g_string_append (system_info, "\n");
@@ -1777,43 +1899,43 @@ main (int argc, char *argv[])
 	guint progress;
 	GtkWidget *main_window;
 	GOptionContext *context;
-	GnomeProgram *program;
 	guint source_id;
 	GError *err = NULL;
 	GtkBuilder *ui = NULL;
 		
 	memset (&gopt_data,  0, sizeof (gopt_data));
 
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (PACKAGE, "UTF-8");
-	textdomain (PACKAGE);
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
 	context = g_option_context_new (N_("\n\nBug Buddy is a utility that helps report debugging\n"
 	                                  "information to the GNOME Bugzilla when a program crashes."));
 
 	g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
-
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
 
-	program = gnome_program_init (PACKAGE, VERSION,
-	                              LIBGNOMEUI_MODULE,
-	                              argc, argv,
-	                              GNOME_PARAM_GOPTION_CONTEXT, context,
-	                              GNOME_PARAM_APP_DATADIR, BUDDY_DATADIR,
- 				      GNOME_CLIENT_PARAM_SM_CONNECT, FALSE,
-	                              NULL);
+	if (!g_option_context_parse (context, &argc, &argv, &err)) {
+		g_critical ("Failed to parse arguments: %s\n", err->message);
+		g_error_free (err);
+		g_option_context_free (context);
+		exit (1);
+        }
+
+	g_option_context_free (context);
+
+	if (!bonobo_activation_is_initialized ())
+		bonobo_activation_init (argc, argv);
 
 	g_set_application_name (_("Bug Buddy"));
 	gtk_window_set_default_icon_name ("bug-buddy");
 
+	s = g_build_filename (BUDDY_DATADIR, "bug-buddy.gtkbuilder", NULL);
 
-	s = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
-				       "bug-buddy.gtkbuilder", TRUE, NULL);
-	if (s) {
-		ui = gtk_builder_new ();
-		gtk_builder_add_from_file (ui, s, &err);
-		gtk_builder_set_translation_domain (ui, GETTEXT_PACKAGE);
-	}
+	ui = gtk_builder_new ();
+	gtk_builder_add_from_file (ui, s, &err);
+	gtk_builder_set_translation_domain (ui, GETTEXT_PACKAGE);
 
 	g_free (s);
 
@@ -1824,7 +1946,6 @@ main (int argc, char *argv[])
 		if (err) {
 			g_error_free (err);
 		}
-		g_object_unref (program);
 		return 0;
 	}
 
@@ -1841,7 +1962,6 @@ main (int argc, char *argv[])
 
 	if (gopt_data.app_file == NULL && gopt_data.package == NULL) {
 		buddy_error (NULL, _("Either --appname or --package arguments are required.\n"));	
-		g_object_unref (program);
 		return 0;
 	}	
 
@@ -1849,9 +1969,13 @@ main (int argc, char *argv[])
             gopt_data.include_file == NULL && 
             gopt_data.minidump_file == NULL) {
 		buddy_error (NULL, _("Either --pid , --include or --minidump arguments are required.\n"));	
-		g_object_unref (program);
 		return 0;
-	}	
+	}
+
+	if (gopt_data.own_file == TRUE && gopt_data.include_file == NULL) {
+		buddy_error (NULL, _("The --unlink-tempfile option needs a --include argument.\n"));
+		return 0;
+	}
 	
 	/* get some information about the gnome version */
 	gnome_version = get_gnome_version_info ();
@@ -1859,7 +1983,6 @@ main (int argc, char *argv[])
 		buddy_error (NULL, _("Bug Buddy was unable to retrieve information regarding "
 		                     "the version of GNOME you are running.  This is most likely "
 		                     "due to a missing installation of gnome-desktop.\n"));
-		g_object_unref (program);
 		return 0;
 	}
 	
@@ -1867,7 +1990,7 @@ main (int argc, char *argv[])
 
 	/* connect the signal handler for the help button */
 	g_signal_connect (gtk_builder_get_object (ui, "help-button"), "clicked", 
-	                  G_CALLBACK (help_callback), program);
+	                  G_CALLBACK (help_callback), NULL);
 
 	gtk_widget_show (main_window);
 
@@ -1916,7 +2039,6 @@ main (int argc, char *argv[])
 				buddy_error (NULL, _("Bug Buddy encountered the following error when trying "
 		                     		     "to retrieve debugging information: %s\n"), err->message);
 				g_error_free (err);
-				g_object_unref (program);
 				return 0;
 			}
 
@@ -1993,20 +2115,13 @@ main (int argc, char *argv[])
 			gtk_entry_set_text (GTK_ENTRY (email_entry), default_email);
 			g_free (default_email);
 		}
-		g_object_set_data (G_OBJECT (ui), "type", GINT_TO_POINTER(BUG_TYPE_REQUEST));
-	}
-	
-	if (gopt_data.include_file != NULL) {
-		fill_include_file (gopt_data.include_file, ui);
+		g_object_set_data (G_OBJECT (ui), "type", GINT_TO_POINTER (BUG_TYPE_REQUEST));
 	}
 
-	
 	g_signal_connect (gtk_builder_get_object (ui, "close-button"), "clicked", 
 	                  G_CALLBACK (close_callback), ui);
-	
-	gtk_main ();
 
-	g_object_unref (program);
+	gtk_main ();
 
 	return 0;
 }
